"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const splToken = __importStar(require("@solana/spl-token"));
const fs_1 = __importDefault(require("fs"));
const mangoAccount_1 = require("../../src/accounts/mangoAccount");
const builder_1 = require("../../src/builder");
const client_1 = require("../../src/client");
const clientIxParamBuilder_1 = require("../../src/clientIxParamBuilder");
const constants_1 = require("../../src/constants");
//
// This script creates liquidation candidates
//
const GROUP_NUM = Number(process.env.GROUP_NUM || 200);
const CLUSTER = process.env.CLUSTER || 'mainnet-beta';
// native prices
const PRICES = {
    ETH: 1200.0,
    SOL: 0.015,
    USDC: 1,
    MNGO: 0.02,
};
const TOKEN_SCENARIOS = [
    [
        'TCS, FUNDING',
        [
            ['USDC', 5000000],
            ['ETH', 100000],
            ['SOL', 150000000],
        ],
        [],
    ],
    ['TCS, LIQOR', [['USDC', 1000000]], []],
    ['TCS, HEALTH', [['USDC', 1000]], []],
    ['TCS, FULL+CLOSE', [['USDC', 1000000]], []],
    ['TCS, EXPIRE', [['USDC', 1000000]], []],
    ['TCS, NET-BORROW', [['USDC', 10000000]], []],
];
async function main() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    options.commitment = 'processed';
    options.preflightCommitment = 'finalized';
    const connection = new web3_js_1.Connection(process.env.CLUSTER_URL, options);
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.PAYER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(admin);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], {
        idsSource: 'get-program-accounts',
        prioritizationFee: 100,
        txConfirmationCommitment: 'confirmed',
    });
    console.log(`User ${userWallet.publicKey.toBase58()}`);
    // fetch group
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    console.log(group.toString());
    const MINTS = new Map([
        ['USDC', group.banksMapByName.get('USDC')[0].mint],
        ['ETH', group.banksMapByName.get('ETH')[0].mint],
        ['SOL', group.banksMapByName.get('SOL')[0].mint],
    ]);
    const accounts = await client.getMangoAccountsForOwner(group, admin.publicKey);
    let maxAccountNum = Math.max(0, ...accounts.map((a) => a.accountNum));
    async function createMangoAccount(name) {
        const accountNum = maxAccountNum + 1;
        maxAccountNum = maxAccountNum + 1;
        await client.createMangoAccount(group, accountNum, name, 4, 4, 4, 4);
        const account = (await client.getMangoAccountForOwner(group, admin.publicKey, accountNum));
        await client.accountExpandV2(group, account, 4, 4, 4, 4, 50);
        await account.reload(client);
        return account;
    }
    async function setBankPrice(bank, price) {
        await client.stubOracleSet(group, bank.oracle, price);
        // reset stable price
        await client.tokenEdit(group, bank.mint, (0, builder_1.Builder)(clientIxParamBuilder_1.NullTokenEditParams).resetStablePrice(true).build());
    }
    async function setPerpPrice(perpMarket, price) {
        await client.stubOracleSet(group, perpMarket.oracle, price);
        // reset stable price
        await client.perpEditMarket(group, perpMarket.perpMarketIndex, (0, builder_1.Builder)(clientIxParamBuilder_1.NullPerpEditParams).resetStablePrice(true).build());
    }
    for (const scenario of TOKEN_SCENARIOS) {
        const [name, assets, liabs] = scenario;
        // create account
        console.log(`Creating mangoaccount...`);
        const mangoAccount = await createMangoAccount(name);
        console.log(`...created mangoAccount ${mangoAccount.publicKey} for ${name}`);
        for (const [assetName, assetAmount] of assets) {
            const assetMint = new web3_js_1.PublicKey(MINTS.get(assetName));
            await client.tokenDepositNative(group, mangoAccount, assetMint, new anchor_1.BN(assetAmount));
            await mangoAccount.reload(client);
        }
        for (const [liabName, liabAmount] of liabs) {
            const liabMint = new web3_js_1.PublicKey(MINTS.get(liabName));
            // temporarily drop the borrowed token value, so the borrow goes through
            const bank = group.banksMapByName.get(liabName)[0];
            try {
                await setBankPrice(bank, PRICES[liabName] / 2);
                await client.tokenWithdrawNative(group, mangoAccount, liabMint, new anchor_1.BN(liabAmount), true);
            }
            finally {
                // restore the oracle
                await setBankPrice(bank, PRICES[liabName]);
            }
        }
    }
    const accounts2 = await client.getMangoAccountsForOwner(group, admin.publicKey);
    // Case LIQEE1: The liqee does not have enough health for the tcs
    {
        const account = ensure(accounts2.find((account) => account.name == 'TCS, HEALTH'));
        await client.tokenConditionalSwapCreateRaw(group, account, MINTS.get('SOL'), MINTS.get('USDC'), new anchor_1.BN(100000000), new anchor_1.BN(20000), // liqee only has 1k USDC-native, leverage does not go that far!
        null, 0.0, 1000000.0, 0.01, true, true, mangoAccount_1.TokenConditionalSwapDisplayPriceStyle.buyTokenPerSellToken, mangoAccount_1.TokenConditionalSwapIntention.unknown);
    }
    // Case LIQEE2: Full execution - tcs closes afterward
    {
        const account = ensure(accounts2.find((account) => account.name == 'TCS, FULL+CLOSE'));
        await client.tokenConditionalSwapCreateRaw(group, account, MINTS.get('SOL'), MINTS.get('USDC'), new anchor_1.BN(1000), new anchor_1.BN(1000), null, 0.0, 1000000.0, 0.01, true, true, mangoAccount_1.TokenConditionalSwapDisplayPriceStyle.buyTokenPerSellToken, mangoAccount_1.TokenConditionalSwapIntention.unknown);
    }
    // Case LIQEE3: Create a tcs that will expire very soon
    {
        const account = ensure(accounts2.find((account) => account.name == 'TCS, EXPIRE'));
        await client.tokenConditionalSwapCreateRaw(group, account, MINTS.get('SOL'), MINTS.get('USDC'), new anchor_1.BN(1000), new anchor_1.BN(1000), Date.now() / 1000 + 15, // expire in 15s
        0.0, 1000000.0, 0.01, true, true, mangoAccount_1.TokenConditionalSwapDisplayPriceStyle.buyTokenPerSellToken, mangoAccount_1.TokenConditionalSwapIntention.unknown);
    }
    // Case LIQEE4: Create a tcs that hits net borrow limits
    {
        const account = ensure(accounts2.find((account) => account.name == 'TCS, NET-BORROW'));
        // To do this, first make a new mint and register it as a new token with tight net borrow limits
        const newMint = await splToken.createMint(connection, admin, admin.publicKey, null, 6);
        const tokenAccount = await splToken.createAssociatedTokenAccountIdempotent(connection, admin, newMint, admin.publicKey);
        await splToken.mintTo(connection, admin, newMint, tokenAccount, admin, 1e15);
        await client.stubOracleCreate(group, newMint, 1.0);
        const newOracle = (await client.getStubOracle(group, newMint))[0];
        const newTokenIndex = Math.max(...group.banksMapByTokenIndex.keys()) + 1;
        await client.tokenRegister(group, newMint, newOracle.publicKey, newTokenIndex, 'TMP', {
            ...clientIxParamBuilder_1.DefaultTokenRegisterParams,
            loanOriginationFeeRate: 0,
            loanFeeRate: 0,
            initAssetWeight: 1,
            maintAssetWeight: 1,
            initLiabWeight: 1,
            maintLiabWeight: 1,
            liquidationFee: 0,
            netBorrowLimitPerWindowQuote: 1500000, // less than the $2 of the tcs
        });
        await group.reloadAll(client);
        await client.tokenConditionalSwapCreateRaw(group, account, newMint, MINTS.get('USDC'), new anchor_1.BN(2000000), // $2
        new anchor_1.BN(2000000), null, 0.0, 1000000.0, 0.01, true, true, mangoAccount_1.TokenConditionalSwapDisplayPriceStyle.buyTokenPerSellToken, mangoAccount_1.TokenConditionalSwapIntention.unknown);
    }
    process.exit();
}
function ensure(value) {
    if (value == null) {
        throw new Error('Value was nullish');
    }
    return value;
}
main();
